#include "midiRecorder.h"
#include "ofxMidi.h"

#include <set>
#include <vector>
#include <math.h>
#include <string>
#include <sstream>
using namespace std;

//--------------------------------------------------------------
void MidiRecorder::setup(){
	
	_midiIn.listPorts();
	_midiIn.openPort(0);
	_midiIn.addListener(this);
	

	//testRecorder();
	
	_franklin_book.loadFont("frabk.ttf", 32);
	
	_master_time = 0;
	
	ofBackground(255, 255, 255);
	
}

//--------------------------------------------------------------
void MidiRecorder::update(){
	_master_time = fmod(ofGetElapsedTimef(), LOOP_SECONDS);
}

//--------------------------------------------------------------
void MidiRecorder::draw(){
	char clockString[255];
	sprintf(clockString, "%3.2f", _master_time);
	
	ofSetColor(0, 0, 0, 255);
	_franklin_book.drawString(clockString, 50, 50);
	
	_franklin_book.drawString(getValues(_master_time).c_str(), 50, 200);
}

//--------------------------------------------------------------
void MidiRecorder::keyPressed(int key){

}

//--------------------------------------------------------------
void MidiRecorder::keyReleased(int key){

}

//--------------------------------------------------------------
void MidiRecorder::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void MidiRecorder::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::windowResized(int w, int h){

}




//--------------------------------------------------------------
void MidiRecorder::testRecorder() {
	// current
	cout << "before manipulation." << endl;
	printExistingIds();
	
	_recorder.setEvent(0, 1, 1);
	_recorder.setEvent(0, 2, 10);
	
	_recorder.setEvent(1, 1, 1);
	
	cout << "0, 1 sould exist." << endl;
	printExistingIds();
	
	_recorder.setEvent(2, 1, 1);
	_recorder.setEvent(2, 2, 2);
	_recorder.setEvent(2, 3, 3);
	
	vector<IdValue> values = _recorder.getValues(0);
	cout << "should be 0|1, 1|1, 2|1" << endl;
	printIdValues(values);

	values = _recorder.getValues(1);
	cout << "should be 0|1, 1|1, 2|1" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(1.5);
	cout << "should be 0|5, 1|1, 2|1.5" << endl;
	printIdValues(values);
	cout << endl << endl;

	values = _recorder.getValues(2);
	cout << "should be 0|10, 1|1, 2|2" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(3);
	cout << "should be 0|10, 1|1, 2|3" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(4);
	cout << "should be 0|10, 1|1, 2|3" << endl;
	printIdValues(values);
	cout << endl << endl;
	
}

void MidiRecorder::printIdValues(const vector<IdValue>& values) {
	vector<IdValue>::const_iterator it;
	cout << "id,values are: ";
	for(it = values.begin(); it != values.end(); ++it) {
		cout << it->id << "|" << it->value << ", ";
	}
	cout << endl;
}

void MidiRecorder::printExistingIds() {
	cout << "existing ids: ";
	cout << returnExistingIdsStr();
}

string MidiRecorder::returnExistingIdsStr() {
	const set<int>& curIds = _recorder.getExistingIds();
	stringstream ss;
	set<int>::iterator it;
	for(it = curIds.begin(); it!=curIds.end(); ++it) {
		ss << *it << ", ";
	}
	ss << endl;
	
	return ss.str();
}

string MidiRecorder::getValues(float time) {
	stringstream ret;
	
	vector<IdValue> values = _recorder.getValues(time);
	vector<IdValue>::iterator it;
	for(it = values.begin(); it != values.end(); ++it) {
		ret << it->id << "/" << it->value << "\n";
	}
	
	return ret.str();
}

void MidiRecorder::newMidiMessage(ofxMidiEventArgs& eventArgs)
{
	cout << "byteOne = " << eventArgs.byteOne << endl;
	cout << "byteTwo = " << eventArgs. byteTwo << endl;
	
	// class is thread safe
	_recorder.setEvent(eventArgs.byteOne, _master_time, eventArgs.byteTwo);
}


